/**
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.siddhi.sample.benchmark.query.types;

import org.apache.log4j.Logger;
import org.siddhi.api.QueryFactory;
import org.siddhi.api.condition.where.ConditionOperator;
import org.siddhi.api.eventstream.InputAbstractEventStream;
import org.siddhi.api.eventstream.query.Query;
import org.siddhi.core.exception.SiddhiException;
import org.siddhi.core.node.CallbackHandler;
import org.siddhi.core.node.InputHandler;
import org.siddhi.core.SiddhiManager;
import org.siddhi.core.event.Event;
import org.siddhi.core.exception.ProcessorInitializationException;
import org.siddhi.core.exception.InvalidQueryException;
import org.siddhi.sample.benchmark.server.RunningTimeStats;

public class SimpleStockQuoteVWAPQueryProvider implements QueryProvider {

    private SiddhiManager siddhiManager;
    private String streamName;
    private InputHandler inputHandler;
    private int queryCount = 0;
    private static final Logger log = Logger.getLogger(SimpleStockQuoteVWAPQueryProvider.class);
    private static long INIT_TIME;
    private boolean isInitialized = false;
    public static long EVENT_COUNTER = 0;
    public static boolean END_RUN = false;
    public static boolean finishedCallBack = false;

    public SimpleStockQuoteVWAPQueryProvider(SiddhiManager siddhiManager, String streamName, int queryCount)
            throws InvalidQueryException, ProcessorInitializationException, SiddhiException {
        this.siddhiManager = siddhiManager;
        this.streamName = streamName;
        this.queryCount = queryCount;
        init();  // initializes Siddhi core and prepare it to process events
    }

    private void init() throws SiddhiException, ProcessorInitializationException,
                               InvalidQueryException {
        System.out.println("... << Initializing SimpleStockQuoteVWAPQueryProvider >> ....");
        InputAbstractEventStream cseEventStream = new InputAbstractEventStream(   // Define the input event stream which maps to the event sender stream definition
                streamName,
                new String[]{"symbol", "price", "volume"},
                new Class[]{String.class, Double.class, Integer.class}
        );

        inputHandler = siddhiManager.addInputEventStream(cseEventStream); // add the input handler to siddhi manager

        QueryFactory qf = siddhiManager.getQueryFactory(); // obtain the query factory to create a given number of queries

        for (int i = 0; i < queryCount; i++) { // creates given number of similar queries from the defined query by only changing the query stock symbol i.e (for 10 queries symbols are S0,S1,..S9)
            String symbol = "S" + i;
            Query query = qf.createQuery(
                    "StockQuoteVWAP",
                    qf.output("symbol=CSEStream.symbol", "price=CSEStream.price", "volume=CSEStream.volume"),
                    qf.from(cseEventStream),
                    qf.condition("CSEStream.symbol", ConditionOperator.EQUAL, symbol)
            );
            siddhiManager.addQuery(query); // add query to siddhi manager

        }

        siddhiManager.addCallback(new CallbackHandler("StockQuoteVWAP") { // define and add the output eventstream of the above query to siddhi manager
            public void callBack(Event event) {
                System.out.println("+ +.........Event captured  " + event + " ++");
                if (RunningTimeStats.EVENT_BASED) {

                    if (((Integer) event.getNthAttribute(2)) == 9999) { // captures the dummy event send from the system to identify whether the eventstream has finished given number of events , and calculate the throughput

                        if (RunningTimeStats.EVENT_BASED_POINTER == RunningTimeStats.EVENT_BASED_SLOTS.length) {
                            RunningTimeStats.addEventSlot(RunningTimeStats.EVENT_BASED_SLOTS[RunningTimeStats.EVENT_BASED_SLOTS.length - 1] + 1,
                                                          System.currentTimeMillis() - RunningTimeStats.EVENT_BASED_TEMPTIMEREF);
                        } else {
                            RunningTimeStats.addEventSlot(RunningTimeStats.EVENT_BASED_SLOTS[RunningTimeStats.EVENT_BASED_POINTER],
                                                          System.currentTimeMillis() - RunningTimeStats.EVENT_BASED_TEMPTIMEREF);
                        }
                        finishedCallBack = true;
                        RunningTimeStats.OVERALL_THROUGHPUT = System.currentTimeMillis() - INIT_TIME;
                    } else if ((RunningTimeStats.EVENT_BASED_POINTER < RunningTimeStats.EVENT_BASED_SLOTS.length)
                               && (EVENT_COUNTER >= RunningTimeStats.EVENT_BASED_SLOTS[RunningTimeStats.EVENT_BASED_POINTER])) {
                        long ms = System.currentTimeMillis();
                        long runtime = ms - RunningTimeStats.EVENT_BASED_TEMPTIMEREF;

                        if (RunningTimeStats.EVENT_BASED_POINTER == 0) {
                            runtime = ms - INIT_TIME;
                        }

                        RunningTimeStats.addEventSlot(RunningTimeStats.EVENT_BASED_SLOTS[RunningTimeStats.EVENT_BASED_POINTER], runtime);
                        RunningTimeStats.EVENT_BASED_POINTER++;
                        RunningTimeStats.EVENT_BASED_TEMPTIMEREF = ms;
                    }

                } else {
                    long ms = System.currentTimeMillis() - SimpleStockQuoteVWAPQueryProvider.INIT_TIME;
                    if (((Integer) event.getNthAttribute(2)) == 9999) {
                        if (RunningTimeStats.POINTER == RunningTimeStats.DEFAULT_MS.length) {
                            RunningTimeStats.addTimeSlot(RunningTimeStats.DEFAULT_MS[RunningTimeStats.DEFAULT_MS.length - 1] + 1, EVENT_COUNTER);
                        } else {
                            RunningTimeStats.addTimeSlot(RunningTimeStats.DEFAULT_MS[RunningTimeStats.POINTER], EVENT_COUNTER);

                        }
                        finishedCallBack = true;
                    } else if ((RunningTimeStats.POINTER < (RunningTimeStats.DEFAULT_MS.length))
                               && (ms >= RunningTimeStats.DEFAULT_MS[RunningTimeStats.POINTER])
                               && (!RunningTimeStats.timeMap.containsValue(EVENT_COUNTER))) {
                        RunningTimeStats.addTimeSlot(RunningTimeStats.DEFAULT_MS[RunningTimeStats.POINTER], EVENT_COUNTER);
                        RunningTimeStats.POINTER++;
                    }

                }
            }

        }
        );

        siddhiManager.update();   //  update siddhi manager of the above new changes


    }

    public void sendEvent(Event event) {  // sends events to the siddhi core which are captured from the server
        if (!isInitialized) {
            INIT_TIME = System.currentTimeMillis();
            isInitialized = true;
        }
        EVENT_COUNTER++;
        inputHandler.sendEvent(event);
    }


}
